<?php

namespace Drupal\personal_digest\Plugin\QueueWorker;

use Drupal\User\Entity\User;
use Drupal\user\UserDataInterface;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Updates a feed's items.
 *
 * @QueueWorker(
 *   id = "personal_digest_mail",
 *   title = @Translation("Generate personalised digest mail"),
 *   cron = {"time" = 60}
 * )
 * @todo inject user.data and mail manager
 */
class DigestMail extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  protected $mailManager;
  protected $userDataStore;

  /**
   * Constructor
   *
   * @param array $configuration
   * @param string $plugin_id
   * @param array $plugin_definition
   */
  public function __construct($configuration, $plugin_id, $plugin_definition, UserDataInterface $user_data, MailManagerInterface $mail_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->userDataStore = $user_data;
    $this->mailManager = $mail_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('user.data'),
      $container->get('plugin.manager.mail')
    );
  }

  /**
   * @param int $data
   *   The user id.
   */
  public function processItem($data) {
    $recipient = User::load($data);
    $settings = $this->userDataStore->get('personal_digest', $data, 'digest');
    // Check the last sending time again in case something got put in the queue
    // twice.
    if ($settings['last'] < strtotime('today')) {
      $this->mailManager
        ->mail('personal_digest',
          'digest',
          $recipient->getEmail(),
          $recipient->getPreferredLangcode(),
          [
            'user' => $recipient,
            'displays'  => $settings['displays'],
            'weeks_interval' => $settings['weeks_interval'],
            'last' => $settings['last'],
          ]
        );
      //the mail is sent, so save the last sent time..
      $settings['last'] = REQUEST_TIME;
      $this->userDataStore->set('personal_digest', $data, 'digest', $settings);
    }

  }

}
